package com.test.s3;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import software.amazon.awssdk.transfer.s3.progress.TransferListener;

@Slf4j
@RequiredArgsConstructor
public class S3TransferListener implements TransferListener {

    final String resource;
    long startTime;
    private int step=0;
    @Override
    public void transferInitiated (Context.TransferInitiated context) {
        log.info("Transfer initiated: {}, {}", resource, context.progressSnapshot().ratioTransferred());
        startTime = System.currentTimeMillis();
        status(context.progressSnapshot().transferredBytes());
    }

    private void status(long l) {
        if (l > step * 1_000_000) {
            log.info("Bytes transferred {}", l);
            step++;
        }
    }

    @Override
    public void bytesTransferred (Context.BytesTransferred context) {
        status(context.progressSnapshot().transferredBytes());
    }

    @Override
    public void transferComplete (Context.TransferComplete context) {
        long seconds = (System.currentTimeMillis() - startTime) / 1000;
        double bytes = (double)context.progressSnapshot().transferredBytes();
        double megabytes = bytes / 1_048_576;
        double throughput = megabytes / seconds;
        log.info("Transfer complete for resource: {}\n\t Bytes: {}\n\t MBs: {}\n\tThroughput: {} MB/s",
                resource, String.format("%10f", bytes), String.format("%.3f", megabytes),
                String.format("%.2f", throughput));
    }

    @Override
    public void transferFailed (Context.TransferFailed context) {
        log.error("Transfer failed on resource "+resource, context.exception());
    }
}