package com.test.s3;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import lombok.extern.slf4j.Slf4j;
import software.amazon.awssdk.auth.credentials.AwsCredentialsProvider;
import software.amazon.awssdk.auth.credentials.ProfileProviderCredentialsContext;
import software.amazon.awssdk.crt.Log;
import software.amazon.awssdk.profiles.Profile;
import software.amazon.awssdk.profiles.ProfileFile;
import software.amazon.awssdk.regions.Region;
import software.amazon.awssdk.services.s3.S3AsyncClient;
import software.amazon.awssdk.services.sso.auth.SsoProfileCredentialsProviderFactory;
import software.amazon.awssdk.transfer.s3.S3TransferManager;

@Slf4j
public class Utils {
    static Path getPath (String resource) {
        Path path = Paths.get(resource);
        if (!path.getParent().toFile().exists() && !path.getParent().toFile().mkdirs()) {
            throw new RuntimeException("Failed to create path location: " + path);
        }
        return path;
    }

    static void initCrtLogging(String path) {
        //Since this method uses native aws libs (https://github.com/awslabs/aws-c-s3)
        // Only decent logging can be found by enabling this.
        Path logFilePath = getPath(path);
        if (Files.exists(logFilePath)) {
            try {
                Files.delete(logFilePath);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }
        Log.initLoggingToFile(Log.LogLevel.Trace, path);
        log.info("CRT Logging initialized: "+path);
    }

    static S3TransferManager getTransferManager(S3AsyncClient s3AsyncClient) {
        return S3TransferManager.builder()
                .s3Client(s3AsyncClient)
                .build();
    }

    public static S3AsyncClient getS3AsyncClient(AwsCredentialsProvider credentialsProvider) {
        S3AsyncClient s3AsyncClient =
                S3AsyncClient.crtBuilder()
                        .credentialsProvider(credentialsProvider)
                        .region(Region.US_EAST_1)
                        .targetThroughputInGbps(20.0)
                        .minimumPartSizeInBytes(1000000L)
                        .build();
        return s3AsyncClient;
    }

    public static AwsCredentialsProvider getSsoCredentials(String profileName) {
        ProfileFile profileFile = ProfileFile.defaultProfileFile();
        Profile profile = profileFile.profile(profileName).get();
        ProfileProviderCredentialsContext profileProvider =
                ProfileProviderCredentialsContext.builder()
                        .profile(profile)
                        .profileFile(profileFile)
                        .build();
        return new SsoProfileCredentialsProviderFactory().create(profileProvider);
    }
}
